<?php
use Drupal\Core\Update\UpdateKernel;
use Symfony\Cmf\Component\Routing\RouteObjectInterface;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;
use Drupal\system\Controller\ApsDbUpdateController;

class Updater {

	/**
	* Execute the non-interactive updater.
	*
	*/
	public function doUpdate($rootDir) {

		// Change the directory to the Drupal root.
		chdir($rootDir);

		$autoloader = require $rootDir . '/autoload.php';
		require $rootDir . '/core/modules/system/system.module';
		define('MAINTENANCE_MODE', 'update');

		$request = Symfony\Component\HttpFoundation\Request::createFromGlobals();
		$kernel = Drupal\Core\DrupalKernel::createFromRequest($request, $autoloader, 'prod', FALSE);
		$kernel->boot();
		$kernel->preHandle($request);

		require __DIR__ . '/ApsDbUpdateController.php';

		$container = $kernel->getContainer();
        // @todo put console/aps/fake renderer to container
        // $container->set('bare_html_page_renderer', $renderer);
		$dbUpdateController = ApsDbUpdateController::create($container);

		$session = $container->get('session');
		$session->start();
		$request->setSession($session);

		// Remove Drupal's error and exception handlers; they rely on a working
		// service container and other subsystems and will only cause a fatal error
		// that hides the actual error.
		restore_error_handler();
		restore_exception_handler();

		drupal_flush_all_caches();

		// Restore Drupal's error and exception handlers.
		// @see \Drupal\Core\DrupalKernel::boot()
		set_error_handler('_drupal_error_handler');
		set_exception_handler('_drupal_exception_handler');

		$batch =& batch_get();
		$batch['progressive'] = false;
		$request->attributes->set(RouteObjectInterface::ROUTE_NAME, 'system.db_update');
		$request->attributes->set(RouteObjectInterface::ROUTE_OBJECT, $container->get('router.route_provider')->getRouteByName('system.db_update'));

        // prevent exception from error decorator
        // @see \Symfony\Component\HttpFoundation\Request::getHost
        $request->server->has('SERVER_ADDR') || $request->server->set('SERVER_ADDR', 'localhost');
        // skip warnings, set $_SESSION['update_ignore_warnings'
        // @see \Drupal\system\Controller\DbUpdateController::handle
        $request->query->get('continue') || $request->query->set('continue', 1);

		$response = $dbUpdateController->handle('run', $request);

        $content = $response->getContent();
        echo $content;
        if ($dbUpdateController->apsSeverity) {
            throw new \Exception($content);
        }

		return $response->getStatusCode() == 200 ? 0 : 1;
	}
}